@@ -13,7 +13,8 @@ from git import Repo
 import logging
 import re
 from urllib.parse import urlparse
-from prettytable import PrettyTable
+from prettytable import PrettyTable, ALL
+import setup
 
 
 # yaml 数据模型
@@ -79,7 +80,7 @@ class ModuleStatusModel:
     branch = ''
     msg = ''
 
-    def __init__(self, module, pod, res, branch, msg):
+    def __init__(self, module, pod, res, branch, msg=''):
         """
         :param module: 模块名字
         :param pod: pod名字
@@ -423,8 +424,6 @@ def update_module_files(yaml_path, local_yaml_path, branch_result, n_branch, mod
     podfile_data = {"version": "1.0.0", "branch": str(n_branch), "dependencies": dependenceList}
     update_yaml(yaml_path, podfile_data)
 
-    print("更新本地yaml")
-    local_dependenceList = []
     after_convert = []
     if not os.path.exists(local_yaml_path):
         shutil.copy(yaml_path, local_yaml_path)  # 复制文件
@@ -432,16 +431,12 @@ def update_module_files(yaml_path, local_yaml_path, branch_result, n_branch, mod
             if mo_re.result == 1:
                 module_dict = {"module": mo_re.module, "pod": mo_re.module, "path": modules_name + "/" + mo_re.module}
                 after_convert.append(module_dict)
-        debugInfo("更新local")
-        debugInfo(after_convert)
     else:
         # 获取ymal 数据
         podfile_module_data = yaml_data(local_yaml_path)
         local_dependenceList = podfile_module_data["dependencies"]
         # 转换成模型数组
         conver_deplist = load_yaml(local_dependenceList)
-        print("转换前")
-        print(conver_deplist)
         for yamlModel in branch_result:
             contain = False
             for cm in conver_deplist:
@@ -460,9 +455,6 @@ def update_module_files(yaml_path, local_yaml_path, branch_result, n_branch, mod
                                          configurations=None,
                                          inhibit_warnings=False)
                 conver_deplist.append(module)
-        print("转换后")
-        print(conver_deplist)
-        # index = 0
         for a in conver_deplist:
             module_dict = {"module": a.module, "pod": a.pod, "path": modules_name + "/" + a.module}
             after_convert.append(module_dict)
@@ -711,8 +703,6 @@ def create_branch(module_list, module_f_path, n_branch, modules, c_path):
     """
     index = 0
     create_branch_result = []
-    print("开始转换了")
-    print(modules)
     for a in module_list:
         if a.module in modules:
             if (a.branch and len(str(a.branch)) > 0) or (a.tag and len(str(a.tag)) > 0):
@@ -720,33 +710,19 @@ def create_branch(module_list, module_f_path, n_branch, modules, c_path):
                 module_path = module_f_path + filename + "/"
                 error_info = ''
                 if not os.path.exists(module_path):
-                    # 没有组件目录，新建分支
-                    print("没有组件目录，新建分支")
                     branch_name = auto_create_branch(module_path, a.git, a.branch, a.tag, n_branch)
                 else:
-                    print("有工作目录就跳过, 打印出当前的分支名")
-                    # 有工作目录就跳过, 打印出当前的分支名
-                    # logging.info("自动创建分支失败 ++++" + a.pod)
                     local_branch = RepoGit(proj_path=module_path).getCurrentBranch()
                     error_info = "本地有工作目录: {0}，当前分支是：{1}".format(a.pod, local_branch)
                     branch_name = ''
                 res = 1
                 if not (branch_name and len(branch_name) > 0):
-
-                    if error_info and len(error_info) > 0:
-                        logging.info(error_info)
-                        print(error_info)
-                    else:
-                        logging.info("自动创建分支失败 ++++" + a.pod)
-                        print("自动创建分支失败 ++++" + a.pod)
-
                     res = 0
-                new_branch_model = YamlBranchModel(a.module, res, a.pod)
+                new_branch_model = ModuleStatusModel(a.module, a.pod, res, n_branch, error_info)
                 create_branch_result.append(new_branch_model)
             else:
-                logging.info("podfileModule.yaml 中组件: " + a.pod + " 的branch或者tag为空, 不能确定要拉取的代码的位置")
-                print("podfileModule.yaml 中组件: " + a.pod + " 的branch或者tag为空, 不能确定要拉取的代码的位置")
-                new_branch_model = YamlBranchModel(a.module, 0, a.pod)
+                new_branch_model = ModuleStatusModel(a.module, a.pod, 0, n_branch,
+                                                     "podfileModule.yaml 中组件: " + a.pod + " 的branch或者tag为空, 不能确定要拉取的代码的位置")
                 create_branch_result.append(new_branch_model)
         index += 1
         os.chdir(c_path)
@@ -843,13 +819,10 @@ def commit_branch(module_list, module_f_path, msg):
         msg = ''
         result = 0
         if not git.is_dirty():
-            msg = "没有可提交的"
+            msg = "很干净，没有可提交的"
         else:
-            debugInfo("开始提交了")
             # 判断是否有没有追踪的文件
             untracks = git.untracked()
-            # debugInfo(untracks)
-            # if len(untracks) > 0:
             git.add(untracks)
             git.commit(msg)
             result = not git.is_dirty()
@@ -911,16 +884,15 @@ def push_branch(module_list, module_f_path):
         git = RepoGit(module_path)
         branch = git.getCurrentBranch()
         msg = ''
-        result = 0
+        result = 1
         if git.is_dirty():
             result = -1
             msg = "本地有变动未提交，请确认"
         else:
-            # ori = git.repo.remotes.origin
             try:
-                git.repo.git.push('--progress', '--no-rebase', 'origin', branch)
+                git.repo.git.push('--progress', 'origin', branch)
             except Exception as e:
-                result = 1
+                result = 0
                 msg = str(e)
 
         modul_branch_model = ModuleStatusModel(module.module, module.pod, result, branch, msg)
@@ -928,12 +900,54 @@ def push_branch(module_list, module_f_path):
     return pull_result
 
 
+# 合并from_branch中代码到当前的开发分支
+def merge_branch(module_list, convert_list, module_f_path, from_branch):
+    """
+    基于列表，提交对应开发分支代码
+    :param module_list: 模块列表
+    :param convert_list: 模块列表
+    :param module_f_path: 路径
+    :param from_branch: 要合并的分支
+    :return: 返回操作的分支
+    """
+    index = 0
+    pull_result = []
+    for module in convert_list:
+        if module.module in module_list:
+            filename = module.module
+            module_path = module_f_path + filename + "/"
+            git = RepoGit(module_path)
+            branch = git.getCurrentBranch()
+            msg = ''
+            result = 1
+            if git.is_dirty():
+                result = -1
+                msg = "本地有变动未提交，请确认"
+            else:
+                branchs = git.get_branches()
+                if from_branch not in branchs:
+                    result = -1
+                    msg = "分支： {0} 不存在".format(from_branch)
+                else:
+                    try:
+                        git.repo.git.merge(from_branch)
+                        git.repo.git.push('--progress', 'origin', branch)
+                    except Exception as e:
+                        result = 0
+                        msg = str(e)
+            modul_branch_model = ModuleStatusModel(module.module, module.pod, result, branch, msg)
+            pull_result.append(modul_branch_model)
+        index += 1
+
+    return pull_result
+
+
 # 基于podfileModule，提交模块开发分支代码
 def release_branch(module_list, tag_path, c_path, f_branch):
     """
     基于模块列表，合并对应开发分支代码到master并打新的tag
     :param module_list: 模块列表
-    :param exception_modules:  这些模块排除在外，不提交n_branch代码
+    :param tag_path:  这些模块排除在外，不提交n_branch代码
     :param c_path: 当前运行分支
     :param f_branch: 配置的全局统一分支，每个模块可以单独配置分支
     :return: 返回操作成功的分支
@@ -949,30 +963,36 @@ def release_branch(module_list, tag_path, c_path, f_branch):
             module_path = tag_path + filename + "/"
             create_tag = auto_release_path(module_path, a.git, a.pod, a.branch, a.new_tag)
             result = 0
-            if not (create_tag and len(create_tag) > 0):
-                print("自动打包失败 ++++" + a.pod)
-            else:
+            if create_tag and len(create_tag) > 0:
                 result = 1
                 a.new_tag = create_tag
-            merge_model = YamlBranchModel(a.module, result, create_tag)
+            merge_model = ModuleStatusModel(a.module, a.pod, result, a.branch)
             merge_result.append(merge_model)
         index += 1
         os.chdir(c_path)
     return merge_result
 
 
+# 打印表格
 def printTable(list, info):
-    error_table = PrettyTable(["编号", "组件名", "分支", "错误信息"])
-    for index in range(0, len(list) - 1):
-        model = list[index]
-        error_table.add_row([index, model.module, model.branch, model.msg])
-    error_table.align[1] = '1'
-    error_table.border = True
-    error_table.junction_char = '+'
-    error_table.horizontal_char = '-'
-    error_table.vertical_char = '|'
-    debugInfo(error_table)
-    debugInfo("[!] 以上是{0}".format(info))
+    table = PrettyTable(["编号", "组件名", "分支", "异常信息"])
+    # debugInfo(len(list))
+    if len(list) > 0:
+        for index in range(len(list)):
+            model = list[index]
+            table.add_row([index, model.module, model.branch, model.msg])
+        table.align[1] = '1'
+        table.border = True
+        table.junction_char = '+'
+        table.horizontal_char = '-'
+        table.vertical_char = '|'
+        table.sortby = '分支'
+        table.hrules = ALL
+        debugInfo("\n[!] 以下是 \"{0}\"\n".format(info))
+        debugInfo(table)
+        debugInfo("\n")
+    else:
+        debugInfo("\n[!] 暂无 \"{0}\"\n".format(info))
 
 
 # 创建分支的对象
@@ -1035,32 +1055,25 @@ class yhgit:
             create_file(modules_path)
         # 获取ymal 数据
         podfile_module_data = yaml_data(yamlPath)
-        logging.info("读取yaml数据")
         # 获取依赖数据
         dependenceList = podfile_module_data["dependencies"]
         # 转换成模型数组
         conver_deplist = load_yaml(dependenceList)
-        logging.info("转换成模型")
         branch_res = create_branch(conver_deplist, modules_path, new_branch, modules, c_path)
-        print("分支转换后的结果")
-        print(branch_res)
         if len(branch_res) > 0:
             update_module_files(yamlPath, localyamlPath, branch_res, new_branch, modules_file_name)
         if len(branch_res) == 0:
-            logging.info("没有要自动创建分支的模块")
-            print("没有要自动创建分支的模块")
+            debugInfo("没有要执行yhgit install的模块")
         else:
-            succ_module = "分支创建成功的模块: \n"
-            fail_module = "分支创建失败的模块: \n"
+            succ_list = []
+            fail_list = []
             for merg_Model in branch_res:
                 if merg_Model.result == 1:
-                    succ_module = succ_module + (merg_Model.module + "\n")
+                    succ_list.append(merg_Model)
                 else:
-                    fail_module = fail_module + (merg_Model.module + "\n")
-            logging.info(succ_module)
-            print(succ_module)
-            print(fail_module)
-            logging.info(fail_module)
+                    fail_list.append(merg_Model)
+            printTable(succ_list, "分支创建成功的模块")
+            printTable(fail_list, "分支创建失败的模块")
 
     # 获取podfileLocal.yaml中各模块状态, 提示： 1. 很干净没有未提交的代码 2. 有改动未提交 3. 没有相关文件报错误
     def status(self):
@@ -1085,7 +1098,6 @@ class yhgit:
         local_dependenceList = podfile_module_data["dependencies"]
         # 转换成模型数组
         module_list = load_yaml(local_dependenceList)
-        debugInfo("开始转换了")
         status_result = []
         for module in module_list:
             result = 0
@@ -1093,7 +1105,7 @@ class yhgit:
             msg = ''
             if not (module.path and len(module.path) > 0):
                 result = -1
-                debugInfo("请检查PodfileLocal.yaml中模块：" + module.module + " 的path")
+                msg = "请检查PodfileLocal.yaml中模块：" + module.module + " 的path"
             else:
                 if os.path.exists(module.path):
                     repogit = RepoGit(module.path)
@@ -1104,42 +1116,27 @@ class yhgit:
                         msg = repogit.getStatusFormatStr
                 else:
                     result = -1
-                    debugInfo("请检查模块：" + module.module + " 的路径 " + module.path + " 是否为空")
-
+                    msg = "请检查模块：" + module.module + " 的路径 " + module.path + " 是否为空";
             new_branch_model = ModuleStatusModel(module.module, module.pod, result, branchname, msg)
             status_result.append(new_branch_model)
 
         if len(status_result) == 0:
             debugInfo("没有要执行git status的模块")
         else:
-            error_module = "检查失败的模块: \n\n"
-            fail_module = "有代码要提交的模块: \n\n"
-            succ_module = "没有代码要提交的模块: \n\n"
             error_list = []
             fail_list = []
             success_list = []
             for merg_Model in status_result:
                 if merg_Model.result == 1:
-                    succ_module = succ_module + (merg_Model.module + " 分支: " + str(merg_Model.branch) + "\n")
                     success_list.append(merg_Model)
                 elif merg_Model.result == 0:
-                    fail_module = fail_module + (
-                            merg_Model.module + " 分支: " + str(merg_Model.branch) + "\n" + " 修改信息: \n" + str(
-                        merg_Model.msg) + "\n\n")
                     fail_list.append(merg_Model)
                 elif merg_Model.result == -1:
                     error_module = error_module + (merg_Model.module + "\n")
                     error_list.append(merg_Model)
-
             printTable(success_list, "没有代码要提交的模块")
             printTable(fail_list, "有代码要提交的模块")
-            printTable(error_module, "检查失败的模块")
-            # logging.info(fail_module)
-            # logging.info(succ_module)
-            # logging.info(error_module)
-            # print(fail_module)
-            # print(succ_module)
-            # print(error_module)
+            printTable(error_list, "检查失败的模块")
 
     # 自动提交本地修改的内容
     def commit(self, msg):
@@ -1153,8 +1150,7 @@ class yhgit:
         modules_path = fa_path + modules_file_name + "/"
         # 子目录，用来存放子模块仓库
         if not os.path.exists(modules_path):
-            logging.info("没有modules目录，不能提交")
-            print("没有modules目录，不能提交")
+            debugInfo("没有modules目录，不能提交")
             return
 
         local_yaml_path = fa_path + 'PodfileLocal.yaml'
@@ -1172,19 +1168,17 @@ class yhgit:
         module_list = load_yaml(local_dependenceList)
         branch_res = commit_branch(module_list, modules_path, msg)
         if len(branch_res) == 0:
-            logging.info("没有要提交代码的模块")
+            debugInfo("没有要提交代码的模块")
         else:
-            succ_module = "代码提交成功的模块: \n"
-            fail_module = "代码提交失败的模块: \n"
+            succ_list = []
+            fail_list = []
             for merg_Model in branch_res:
                 if merg_Model.result == 1:
-                    succ_module = succ_module + (merg_Model.module + " 分支: " + str(merg_Model.branch) + "\n")
+                    succ_list.append(merg_Model)
                 else:
-                    fail_module = fail_module + (merg_Model.module + " 分支: " + str(merg_Model.branch) + "\n")
-                    if merg_Model.msg and len(merg_Model.msg) > 0:
-                        fail_module = fail_module + " 错误原因: " + merg_Model.msg + "\n"
-            debugInfo(fail_module)
-            debugInfo(succ_module)
+                    fail_list.append(merg_Model)
+            printTable(succ_list, "代码提交成功的模块")
+            printTable(fail_list, "代码提交失败的模块")
 
     # 自动拉取远端代码
     def pull(self):
@@ -1215,17 +1209,15 @@ class yhgit:
         if len(branch_res) == 0:
             logging.info("没有要拉取远端代码的模块")
         else:
-            succ_module = "代码拉取成功的模块: \n"
-            fail_module = "代码拉取失败的模块: \n"
+            succ_list = []
+            fail_list = []
             for merg_Model in branch_res:
                 if merg_Model.result == 1:
-                    succ_module = succ_module + (merg_Model.module + " 分支: " + str(merg_Model.branch) + "\n")
+                    succ_list.append(merg_Model)
                 else:
-                    fail_module = fail_module + (merg_Model.module + " 分支: " + str(merg_Model.branch) + "\n")
-                    if merg_Model.msg and len(merg_Model.msg) > 0:
-                        fail_module = fail_module + " 错误原因: " + merg_Model.msg + "\n"
-            debugInfo(fail_module)
-            debugInfo(succ_module)
+                    fail_list.append(merg_Model)
+            printTable(succ_list, "代码拉取成功的模块")
+            printTable(fail_list, "代码拉取失败的模块")
 
     # 自动推送本地修改到远端
     def push(self):
@@ -1256,17 +1248,52 @@ class yhgit:
         if len(branch_res) == 0:
             logging.info("没有要推送代码的模块")
         else:
-            succ_module = "代码推送成功的模块: \n"
-            fail_module = "代码推送失败的模块: \n"
+            succ_list = []
+            fail_list = []
             for merg_Model in branch_res:
                 if merg_Model.result == 1:
-                    succ_module = succ_module + (merg_Model.module + " 分支: " + str(merg_Model.branch) + "\n")
+                    succ_list.append(merg_Model)
                 else:
-                    fail_module = fail_module + (merg_Model.module + " 分支: " + str(merg_Model.branch) + "\n")
-                    if merg_Model.msg and len(merg_Model.msg) > 0:
-                        fail_module = fail_module + " 错误原因: " + merg_Model.msg + "\n"
-            debugInfo(fail_module)
-            debugInfo(succ_module)
+                    fail_list.append(merg_Model)
+            printTable(succ_list, "代码推送成功的模块")
+            printTable(fail_list, "代码推送失败的模块")
+
+    # 将from_branch 中的代码合并到当前的开发分支
+    def merge(self, from_branch, module_list):
+        """
+                基于PodfileLocal的配置信息，拉取本地开发分支代码
+                :return:
+                """
+        fa_path = ""
+        modules_file_name = "modules"
+        modules_path = fa_path + modules_file_name + "/"
+        # 子目录，用来存放子模块仓库
+        if not os.path.exists(modules_path):
+            logging.info("没有modules目录，不能提交")
+            print("没有modules目录，不能提交")
+            return
+
+        yamlPath = fa_path + 'PodfileModule.yaml'
+        # 获取ymal 数据
+        podfile_module_data = yaml_data(yamlPath)
+        # 获取依赖数据
+        dependenceList = podfile_module_data["dependencies"]
+        # 转换成模型数组
+        conver_deplist = load_yaml(dependenceList)
+
+        branch_res = merge_branch(module_list, conver_deplist, modules_path, from_branch)
+        if len(branch_res) == 0:
+            logging.info("没有要merge代码的模块")
+        else:
+            succ_list = []
+            fail_list = []
+            for merg_Model in branch_res:
+                if merg_Model.result == 1:
+                    succ_list.append(merg_Model)
+                else:
+                    fail_list.append(merg_Model)
+            printTable(succ_list, "代码merge成功的模块")
+            printTable(fail_list, "代码merge失败的模块")
 
     # 自动提交子模块的代码
     def release(self):
@@ -1302,15 +1329,16 @@ class yhgit:
         if len(branch_res) == 0:
             logging.info("没有要realse的模块")
         else:
-            succ_module = "release成功的模块: \n"
-            fail_module = "realse失败的模块: \n"
+            succ_list = []
+            fail_list = []
             for merg_Model in branch_res:
                 if merg_Model.result == 1:
-                    succ_module = succ_module + (merg_Model.module + "\n")
+                    succ_list.append(merg_Model)
                 else:
-                    fail_module = fail_module + (merg_Model.module + "\n")
-            logging.info(succ_module)
-            logging.info(fail_module)
+                    fail_list.append(merg_Model)
+            printTable(succ_list, "release成功的模块")
+            printTable(fail_list, "realse失败的模块")
+
         if os.path.exists(modules_path):
             del_file(modules_path)
 
@@ -1328,6 +1356,8 @@ class yhgit:
         if os.path.exists(modules_path):
             del_file(modules_path)
 
+        debugInfo("执行成功")
+
 
 def debugInfo(msg):
     logging.info(msg)
@@ -1337,22 +1367,13 @@ def debugInfo(msg):
 def main(argvs=None):
     """The main routine."""
 
-    # Do argument parsing here (eg. with argparse) and anything else
-    # you want your project to do.
-    # 获取参数
-    print(argvs)
     if argvs is None:
         argvs = sys.argv
-    print(argvs)
     # 获取指令类型
     command = argvs[1]
     branchname = ''
     modules = []
-    debugInfo(len(argvs))
-    debugInfo(command)
-    print(command == "clean")
     if command == "install":
-        print(len(argvs))
         if len(argvs) > 2:
             subcommand = argvs[2]
             if subcommand and len(subcommand) > 0 and subcommand == "-b":
@@ -1380,9 +1401,25 @@ def main(argvs=None):
             logging.error("请指定具体的模块")
         else:
             yhgit().install(branchname, modules)
+    elif command == "merge":
+        if len(argvs) > 2:
+            subcommand = argvs[2]
+            if subcommand and len(subcommand) > 0 and subcommand == "-b":
+                # 需要获取具体的分支名
+                branchname = argvs[3]
+                if branchname and len(branchname) > 0:
+                    # 获取需要创建分支的组件
+                    modules = argvs[4:]
+        if not (branchname and len(branchname) > 0):
+            debugInfo("必须指定merge的分支: -b 分支")
+            exit()
+        if not (modules and len(modules) > 0):
+            logging.error("请指定具体的模块")
+        else:
+            yhgit().merge(branchname, modules)
+
     elif command == "status":
         # 查看本地代码的状态
-        print("进入这里面了")
         yhgit().status()
     elif command == "commit":
         commit_msg = ''
@@ -1395,7 +1432,6 @@ def main(argvs=None):
             debugInfo("必须添加-m 和 commit_msg")
         else:
             yhgit().commit(commit_msg)
-
     elif command == "pull":
         yhgit().pull()
 
@@ -1407,6 +1443,44 @@ def main(argvs=None):
 
     elif command == "clean":
         yhgit().clean()
+    elif command == "version":
+        debugInfo(setup.version)
+    elif command == "help" or command == "-h":
+
+        """
+            文档指令
+            0. install 组件  根据本地PodfileLocal.yaml中的配置，获取新分支branch，然后基于的组件新建分支branch
+            1. install -b branch 组件 根据分支branch，然后基于组件新建分支branch
+            2. status 根据本地PodfileLocal.yaml中配置的仓库，获取所有仓库的开发分支及仓库状态
+            3. commit 根据本地PodfileLocal.yaml中配置的仓库，提交本地仓库中的修改
+            4. pull   根据本地PodfileLocal.yaml中配置的仓库，拉取远端仓库中代码
+            5. merge   根据本地PodfileLocal.yaml中配置的仓库，然后基于组件merge 分支到当前开发分支，并自动提交
+            6. push   根据本地PodfileLocal.yaml中配置的仓库，推送本地代码到远端
+            5. push   根据本地PodfileLocal.yaml中配置的仓库，推送本地代码到远端
+
+        """
+
+        table = PrettyTable(["编号", "指令", "参数", "说明"])
+        table.add_row(
+            [0, "install", "-b 指定分支名; 组件 指定 新建分支的组件", "根据分支branch，然后基于组件新建分支branch"])
+        table.add_row([1, "install", "组件 指定 新建分支的组件",
+                       "根据本地PodfileLocal.yaml中的配置，获取新分支branch，然后基于的组件新建分支branch"])
+        table.add_row([2, "status", "", "根据本地PodfileLocal.yaml中配置的仓库，获取所有仓库的开发分支及仓库状态"])
+        table.add_row([3, "commit", "-m 指定提交的信息", "根据本地PodfileLocal.yaml中配置的仓库，提交本地仓库中的修改"])
+        table.add_row([4, "pull", "", "根据本地PodfileLocal.yaml中配置的仓库，拉取远端仓库中代码"])
+        table.add_row([5, "push", "", "根据本地PodfileLocal.yaml中配置的仓库，推送本地代码到远端"])
+        table.add_row(
+            [5, "merge", "-b 指定分支名; 组件 指定 merge的组件", "然后基于组件merge 分支到当前开发分支，并自动提交"])
+        table.add_row([6, "release", "", "根据本地PodfileModule.yaml中配置的仓库，合并开发分支到master，并自动打新的tag"])
+        table.add_row([7, "clean", "", "清空本地PodfileLocal.yaml 及 modules文件夹"])
+        table.align[1] = '1'
+        table.border = True
+        table.junction_char = '+'
+        table.horizontal_char = '-'
+        table.vertical_char = '|'
+        table.hrules = ALL
+        debugInfo(table)
+        debugInfo("\n")
 
 
 if __name__ == '__main__':
